@@ -66,6 +66,10 @@ extern GSList* all_panels;
 extern gchar *cprofile;
 extern int config;
 
+/* GtkColotButton expects a number between 0 and 65535, but p->alpha has range
+ * 0 to 255, and (2^(2n) - 1) / (2^n - 1) = 2^n + 1 = 257, with n = 8. */
+static guint16 const alpha_scale_factor = 257;
+
 void panel_global_config_save( Panel* p, FILE *fp);
 void panel_plugin_config_save( Panel* p, FILE *fp);
 
@@ -237,6 +241,15 @@ static void set_width_type( GtkWidget *item, Panel* p )
     update_panel_geometry(p);
 }
 
+static void set_log_level( GtkWidget *cbox, Panel* p)
+{
+    configured_log_level = gtk_combo_box_get_active(GTK_COMBO_BOX(cbox));
+    if (!log_level_set_on_commandline)
+        log_level = configured_log_level;
+    ERR("panel-pref: log level configured to %d, log_level is now %d\n",
+            configured_log_level, log_level);
+}
+
 static void transparency_toggle( GtkWidget *b, Panel* p)
 {
     GtkWidget* tr = (GtkWidget*)g_object_get_data(G_OBJECT(b), "tint_clr");
@@ -320,7 +333,7 @@ on_tint_color_set( GtkColorButton* clr,  Panel* p )
 {
     gtk_color_button_get_color( clr, &p->gtintcolor );
     p->tintcolor = gcolor2rgb24(&p->gtintcolor);
-    p->alpha = gtk_color_button_get_alpha( clr ) / 256;
+    p->alpha = gtk_color_button_get_alpha( clr ) / alpha_scale_factor;
     panel_update_background( p );
 }
 
@@ -336,6 +349,26 @@ on_use_font_color_toggled( GtkToggleButton* btn,   Panel* p )
     panel_set_panel_configuration_changed(p);
 }
 
+static void
+on_font_size_set( GtkSpinButton* spin, Panel* p )
+{
+    p->fontsize = (int)gtk_spin_button_get_value(spin);
+    panel_set_panel_configuration_changed(p);
+}
+
+static void
+on_use_font_size_toggled( GtkToggleButton* btn,   Panel* p )
+{
+    GtkWidget* clr = (GtkWidget*)g_object_get_data( G_OBJECT(btn), "clr" );
+    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(btn)))
+        gtk_widget_set_sensitive( clr, TRUE );
+    else
+        gtk_widget_set_sensitive( clr, FALSE );
+    p->usefontsize = gtk_toggle_button_get_active( btn );
+    panel_set_panel_configuration_changed(p);
+}
+
+
 static void
 set_dock_type(GtkToggleButton* toggle,  Panel* p )
 {
@@ -483,6 +516,23 @@ static void init_plugin_list( Panel* p, GtkTreeView* view, GtkWidget* label )
         gtk_tree_selection_select_iter( tree_sel, &it );
 }
 
+static void on_add_plugin_row_activated( GtkTreeView *tree_view, 
+                                         GtkTreePath *path, 
+                                         GtkTreeViewColumn *col, 
+                                         gpointer user_data) 
+{
+    GtkWidget *dlg;
+
+    dlg = (GtkWidget *) user_data; 
+
+    (void) tree_view;
+    (void) path;
+    (void) col;
+
+    /* Emitting the "response" signal ourselves. */
+    gtk_dialog_response(GTK_DIALOG(dlg), GTK_RESPONSE_OK);
+}
+
 static void on_add_plugin_response( GtkDialog* dlg,
                                     int response,
                                     GtkTreeView* _view )
@@ -611,8 +661,15 @@ static void on_add_plugin( GtkButton* btn, GtkTreeView* _view )
     gtk_tree_view_set_model( view, GTK_TREE_MODEL(list) );
     g_object_unref( list );
 
+    /* 
+     * The user can add a plugin either by clicking the "Add" button, or by
+     * double-clicking the plugin.
+     */
     g_signal_connect( dlg, "response",
                       G_CALLBACK(on_add_plugin_response), _view );
+    g_signal_connect( view, "row-activated",
+                      G_CALLBACK(on_add_plugin_row_activated), (gpointer) dlg);
+
     g_object_set_data( G_OBJECT(dlg), "avail-plugins", view );
     g_object_weak_ref( G_OBJECT(dlg), (GWeakNotify) plugin_class_list_free, classes );
 
@@ -924,7 +981,7 @@ void panel_configure( Panel* p, int sel_page )
     /* transparancy */
     tint_clr = w = (GtkWidget*)gtk_builder_get_object( builder, "tint_clr" );
     gtk_color_button_set_color((GtkColorButton*)w, &p->gtintcolor);
-    gtk_color_button_set_alpha((GtkColorButton*)w, 256 * p->alpha);
+    gtk_color_button_set_alpha((GtkColorButton*)w, alpha_scale_factor * p->alpha);
     if ( ! p->transparent )
         gtk_widget_set_sensitive( w, FALSE );
     g_signal_connect( w, "color-set", G_CALLBACK( on_tint_color_set ), p );
@@ -952,7 +1009,7 @@ void panel_configure( Panel* p, int sel_page )
         w = (GtkWidget*)gtk_builder_get_object( builder, "img_file" );
         g_object_set_data(G_OBJECT(img), "img_file", w);
         gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(w),
-            ((p->background_file != NULL) ? p->background_file : PACKAGE_DATA_DIR "/lxpanel/images/background.png"));
+            ((p->background_file != NULL) ? p->background_file : gtk_icon_info_get_filename(gtk_icon_theme_lookup_icon(p->icon_theme, "lxpanel-backkground", 0, 0))));
 
         if (!p->background)
             gtk_widget_set_sensitive( w, FALSE);
@@ -972,6 +1029,19 @@ void panel_configure( Panel* p, int sel_page )
     if( ! p->usefontcolor )
         gtk_widget_set_sensitive( w, FALSE );
 
+    /* font size */
+    w = (GtkWidget*)gtk_builder_get_object( builder, "font_size" );
+    gtk_spin_button_set_value( (GtkSpinButton*)w, p->fontsize );
+    g_signal_connect( w, "value-changed",
+                      G_CALLBACK(on_font_size_set), p);
+
+    w2 = (GtkWidget*)gtk_builder_get_object( builder, "use_font_size" );
+    gtk_toggle_button_set_active( (GtkToggleButton*)w2, p->usefontsize );
+    g_object_set_data( G_OBJECT(w2), "clr", w );
+    g_signal_connect(w2, "toggled", G_CALLBACK(on_use_font_size_toggled), p);
+    if( ! p->usefontsize )
+        gtk_widget_set_sensitive( w, FALSE );
+
     /* plugin list */
     {
         GtkWidget* plugin_list = (GtkWidget*)gtk_builder_get_object( builder, "plugin_list" );
@@ -1024,6 +1094,11 @@ void panel_configure( Panel* p, int sel_page )
                         &logout_cmd);
     }
 
+    w = (GtkWidget*)gtk_builder_get_object( builder, "log_level" );
+    update_opt_menu(w, configured_log_level);
+    g_signal_connect(w, "changed", G_CALLBACK(set_log_level), p);
+
+
     panel_adjust_geometry_terminology(p);
     gtk_widget_show(GTK_WIDGET(p->pref_dialog));
     w = (GtkWidget*)gtk_builder_get_object( builder, "notebook" );
@@ -1052,10 +1127,13 @@ panel_global_config_save( Panel* p, FILE *fp)
     lxpanel_put_bool(fp, "setdocktype", p->setdocktype);
     lxpanel_put_bool(fp, "setpartialstrut", p->setstrut);
     lxpanel_put_bool(fp, "usefontcolor", p->usefontcolor);
+    lxpanel_put_int(fp, "fontsize", p->fontsize);
     lxpanel_put_line(fp, "fontcolor=#%06x", gcolor2rgb24(&p->gfontcolor));
+    lxpanel_put_bool(fp, "usefontsize", p->usefontsize);
     lxpanel_put_bool(fp, "background", p->background );
     lxpanel_put_str(fp, "backgroundfile", p->background_file);
     lxpanel_put_int(fp, "iconsize", p->icon_size);
+    lxpanel_put_int(fp, "loglevel", configured_log_level);
     lxpanel_put_line(fp, "}\n");
 }
 
@@ -1254,7 +1332,7 @@ GtkWidget* create_generic_config_dlg( const char* title, GtkWidget* parent,
         GtkWidget* label = gtk_label_new( name );
         GtkWidget* entry = NULL;
         gpointer val = va_arg( args, gpointer );
-        GType type = va_arg( args, GType );
+        enum _PluginConfType type = va_arg( args, enum _PluginConfType );
         switch( type )
         {
             case CONF_TYPE_STR:
@@ -1315,8 +1393,14 @@ GtkWidget* create_generic_config_dlg( const char* title, GtkWidget* parent,
                     gtk_box_pack_start( GTK_BOX(hbox), browse, TRUE, TRUE, 2 );
                     g_object_set_data(G_OBJECT(dlg), "file-val", val);
                     g_object_set_data(G_OBJECT(browse), "dlg", dlg);
-                    g_object_set_data(G_OBJECT(browse), "chooser-action",
-                        (gpointer) ((type == CONF_TYPE_DIRECTORY_ENTRY) ? GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER : GTK_FILE_CHOOSER_ACTION_OPEN));
+                    
+                    GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_OPEN;
+                    if (type == CONF_TYPE_DIRECTORY_ENTRY)
+                    {
+                      action = GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER;
+                    }
+
+                    g_object_set_data(G_OBJECT(browse), "chooser-action", GINT_TO_POINTER(action));
                     g_signal_connect( browse, "clicked", G_CALLBACK(on_browse_btn_clicked), entry );
                 }
             }
