@@ -106,8 +106,11 @@ static Panel* panel_allocate(void)
     p->tintcolor = gcolor2rgb24(&p->gtintcolor);
     p->usefontcolor = 0;
     p->fontcolor = 0x00000000;
+    p->usefontsize = 0;
+    p->fontsize = 10;
     p->spacing = 0;
     p->icon_size = PANEL_ICON_SIZE;
+    p->icon_theme = gtk_icon_theme_get_default();
     return p;
 }
 
@@ -419,7 +422,7 @@ void panel_determine_background_pixmap(Panel * p, GtkWidget * widget, GdkWindow
         }
         pixmap = fb_bg_get_xroot_pix_for_win(p->bg, widget);
         if ((pixmap != NULL) && (pixmap != GDK_NO_BG) && (p->alpha != 0))
-            fb_bg_composite(pixmap, widget->style->black_gc, p->tintcolor, p->alpha);
+            fb_bg_composite(pixmap, &p->gtintcolor, p->alpha);
     }
 
     if (pixmap != NULL)
@@ -484,13 +487,19 @@ panel_size_req(GtkWidget *widget, GtkRequisition *req, Panel *p)
 {
     ENTER;
 
+    if (p->autohide && !p->visible)
+        /* When the panel is in invisible state, the content box also got hidden, thus always
+         * report 0 size.  Ask the content box instead for its size. */
+        gtk_widget_size_request(p->box, req);
+
     if (p->widthtype == WIDTH_REQUEST)
         p->width = (p->orientation == ORIENT_HORIZ) ? req->width : req->height;
     if (p->heighttype == HEIGHT_REQUEST)
         p->height = (p->orientation == ORIENT_HORIZ) ? req->height : req->width;
+
     calculate_position(p);
-    req->width  = p->aw;
-    req->height = p->ah;
+
+    gtk_widget_set_size_request( widget, p->aw, p->ah );
 
     RET( TRUE );
 }
@@ -578,8 +587,8 @@ static void panel_popupmenu_remove_item( GtkMenuItem* item, Plugin* plugin )
     }
 
     panel->plugins = g_list_remove( panel->plugins, plugin );
-    plugin_delete(plugin);
     panel_config_save( plugin->panel );
+    plugin_delete(plugin);
 }
 
 /* FIXME: Potentially we can support multiple panels at the same edge,
@@ -687,7 +696,23 @@ static void panel_popupmenu_about( GtkMenuItem* item, Panel* panel )
     panel_apply_icon(GTK_WINDOW(about));
     gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about), VERSION);
     gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about), _("LXPanel"));
-    gtk_about_dialog_set_logo(GTK_ABOUT_DIALOG(about), gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/lxpanel/images/my-computer.png", NULL));
+
+    if(gtk_icon_theme_has_icon(panel->icon_theme, "video-display"))
+    {
+         gtk_about_dialog_set_logo( GTK_ABOUT_DIALOG(about),
+                                    gtk_icon_theme_load_icon(panel->icon_theme, "video-display", 48, 0, NULL));
+    }
+    else if (gtk_icon_theme_has_icon(panel->icon_theme, "start-here"))
+    {
+         gtk_about_dialog_set_logo( GTK_ABOUT_DIALOG(about),
+                                    gtk_icon_theme_load_icon(panel->icon_theme, "start-here", 48, 0, NULL));
+    }
+    else 
+    {
+        gtk_about_dialog_set_logo(  GTK_ABOUT_DIALOG(about),
+                                    gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/lxpanel/images/my-computer.png", NULL));
+    }
+
     gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about), _("Copyright (C) 2008-2011"));
     gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about), _( "Desktop panel for LXDE project"));
     gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about), "This program is free software; you can redistribute it and/or\nmodify it under the terms of the GNU General Public License\nas published by the Free Software Foundation; either version 2\nof the License, or (at your option) any later version.\n\nThis program is distributed in the hope that it will be useful,\nbut WITHOUT ANY WARRANTY; without even the implied warranty of\nMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\nGNU General Public License for more details.\n\nYou should have received a copy of the GNU General Public License\nalong with this program; if not, write to the Free Software\nFoundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.");
@@ -700,7 +725,21 @@ static void panel_popupmenu_about( GtkMenuItem* item, Panel* panel )
 
 void panel_apply_icon( GtkWindow *w )
 {
-    gtk_window_set_icon_from_file(w, PACKAGE_DATA_DIR "/lxpanel/images/my-computer.png", NULL);
+	GdkPixbuf* window_icon;
+	
+	if(gtk_icon_theme_has_icon(gtk_icon_theme_get_default(), "video-display"))
+    {
+		window_icon = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "video-display", 24, 0, NULL);
+	}
+	else if(gtk_icon_theme_has_icon(gtk_icon_theme_get_default(), "start-here"))
+    {
+		window_icon = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), "start-here", 24, 0, NULL);
+	}
+    else
+    {
+		window_icon = gdk_pixbuf_new_from_file(PACKAGE_DATA_DIR "/lxpanel/images/my-computer.png", NULL);
+	}
+    gtk_window_set_icon(w, window_icon);
 }
 
 GtkMenu* lxpanel_get_panel_menu( Panel* panel, Plugin* plugin, gboolean use_sub_menu )
@@ -906,7 +945,7 @@ void panel_establish_autohide(Panel *p)
 }
 
 /* Set an image from a file with scaling to the panel icon size. */
-void panel_image_set_from_file(Panel * p, GtkWidget * image, char * file)
+void panel_image_set_from_file(Panel * p, GtkWidget * image, const char * file)
 {
     GdkPixbuf * pixbuf = gdk_pixbuf_new_from_file_at_scale(file, p->icon_size, p->icon_size, TRUE, NULL);
     if (pixbuf != NULL)
@@ -919,9 +958,9 @@ void panel_image_set_from_file(Panel * p, GtkWidget * image, char * file)
 /* Set an image from a icon theme with scaling to the panel icon size. */
 gboolean panel_image_set_icon_theme(Panel * p, GtkWidget * image, const gchar * icon)
 {
-    if (gtk_icon_theme_has_icon(gtk_icon_theme_get_default(), icon))
+    if (gtk_icon_theme_has_icon(p->icon_theme, icon))
     {
-        GdkPixbuf * pixbuf = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(), icon, p->icon_size, 0, NULL);
+        GdkPixbuf * pixbuf = gtk_icon_theme_load_icon(p->icon_theme, icon, p->icon_size, 0, NULL);
         gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
         g_object_unref(pixbuf);
         return TRUE;
@@ -1065,12 +1104,17 @@ void panel_draw_label_text(Panel * p, GtkWidget * label, char * text, gboolean b
     {
         /* Compute an appropriate size so the font will scale with the panel's icon size. */
         int font_desc;
-        if (p->icon_size < 20) 
-            font_desc = 9;
-        else if (p->icon_size >= 20 && p->icon_size < 36)
-            font_desc = 10;
-        else
-            font_desc = 12;
+        if (p->usefontsize)
+        	font_desc = p->fontsize;
+        else 
+        {
+        	if (p->icon_size < 20) 
+        		font_desc = 9;
+        	else if (p->icon_size >= 20 && p->icon_size < 36)
+        		font_desc = 10;
+        	else
+        		font_desc = 12;
+        }
 
         /* Check the string for characters that need to be escaped.
          * If any are found, create the properly escaped string and use it instead. */
@@ -1229,12 +1273,20 @@ panel_parse_global(Panel *p, char **fp)
                         gdk_color_parse ("black", &p->gfontcolor);
                     p->fontcolor = gcolor2rgb24(&p->gfontcolor);
                     DBG("fontcolor=%x\n", p->fontcolor);
+                } else if (!g_ascii_strcasecmp(s.t[0], "useFontSize")) {
+                    p->usefontsize = str2num(bool_pair, s.t[1], 0);
+                } else if (!g_ascii_strcasecmp(s.t[0], "FontSize")) {
+                    p->fontsize = atoi(s.t[1]);                    
                 } else if (!g_ascii_strcasecmp(s.t[0], "Background")) {
                     p->background = str2num(bool_pair, s.t[1], 0);
                 } else if( !g_ascii_strcasecmp(s.t[0], "BackgroundFile") ) {
                     p->background_file = g_strdup( s.t[1] );
                 } else if (!g_ascii_strcasecmp(s.t[0], "IconSize")) {
                     p->icon_size = atoi(s.t[1]);
+                } else if (!g_ascii_strcasecmp(s.t[0], "LogLevel")) {
+                    configured_log_level = atoi(s.t[1]);
+                    if (!log_level_set_on_commandline)
+                        log_level = configured_log_level;
                 } else {
                     ERR( "lxpanel: %s - unknown var in Global section\n", s.t[0]);
                 }
@@ -1584,6 +1636,7 @@ int main(int argc, char *argv[], char *env[])
 
 	g_thread_init(NULL);
 	gdk_threads_init();
+	gdk_threads_enter();
 
     gtk_init(&argc, &argv);
 
@@ -1616,6 +1669,7 @@ int main(int argc, char *argv[], char *env[])
                 exit(1);
             } else {
                 log_level = atoi(argv[i]);
+                log_level_set_on_commandline = true;
             }
         } else if (!strcmp(argv[i], "--configure") || !strcmp(argv[i], "-C")) {
             config = 1;
@@ -1642,8 +1696,7 @@ int main(int argc, char *argv[], char *env[])
     }
 
     /* Add our own icons to the search path of icon theme */
-    gtk_icon_theme_append_search_path( gtk_icon_theme_get_default(),
-                                       PACKAGE_DATA_DIR "/lxpanel/images" );
+    gtk_icon_theme_append_search_path( gtk_icon_theme_get_default(), PACKAGE_DATA_DIR "/lxpanel/images" );
 
     fbev = fb_ev_new();
     win_grp = gtk_window_group_new();
@@ -1682,6 +1735,8 @@ restart:
     if( is_restarting )
         goto restart;
 
+    gdk_threads_leave();
+
     g_object_unref(win_grp);
     g_object_unref(fbev);
 
