@@ -44,6 +44,7 @@ typedef struct {
     char * action;				/* Command to execute on a click */
     gboolean bold;				/* True if bold font */
     gboolean icon_only;				/* True if icon only (no clock value) */
+    gboolean center_text;
     guint timer;				/* Timer for periodic update */
     enum {
 	AWAITING_FIRST_CHANGE,			/* Experimenting to determine interval, waiting for first change */
@@ -311,6 +312,8 @@ static int dclock_constructor(Plugin * p, char ** fp)
                     dc->bold = str2num(bool_pair, s.t[1], 0);
                 else if (g_ascii_strcasecmp(s.t[0], "IconOnly") == 0)
                     dc->icon_only = str2num(bool_pair, s.t[1], 0);
+                else if (g_ascii_strcasecmp(s.t[0], "CenterText") == 0)
+                    dc->center_text = str2num(bool_pair, s.t[1], 0);
                 else
                     ERR( "dclock: unknown var %s\n", s.t[0]);
             }
@@ -326,7 +329,7 @@ static int dclock_constructor(Plugin * p, char ** fp)
     p->pwid = gtk_event_box_new();
 
     /* Allocate a horizontal box as the child of the top level. */
-    GtkWidget * hbox = gtk_hbox_new(FALSE, 0);
+    GtkWidget * hbox = gtk_hbox_new(TRUE, 0);
     gtk_container_add(GTK_CONTAINER(p->pwid), hbox);
     gtk_widget_show(hbox);
 
@@ -344,9 +347,9 @@ static int dclock_constructor(Plugin * p, char ** fp)
 
     /* Initialize the clock display. */
     if (dc->clock_format == NULL)
-        dc->clock_format = g_strdup(DEFAULT_CLOCK_FORMAT);
+        dc->clock_format = g_strdup(_(DEFAULT_CLOCK_FORMAT));
     if (dc->tooltip_format == NULL)
-        dc->tooltip_format = g_strdup(DEFAULT_TIP_FORMAT);
+        dc->tooltip_format = g_strdup(_(DEFAULT_TIP_FORMAT));
     dclock_apply_configuration(p);
 
     /* Show the widget and return. */
@@ -384,7 +387,9 @@ static void dclock_apply_configuration(Plugin * p)
     /* Set up the icon or the label as the displayable widget. */
     if (dc->icon_only)
     {
-        panel_image_set_from_file(p->panel, dc->clock_icon, PACKAGE_DATA_DIR "/lxpanel/images/clock.png");
+		if(panel_image_set_icon_theme(p->panel, dc->clock_icon, "clock") != FALSE) {
+			panel_image_set_from_file(p->panel, dc->clock_icon, PACKAGE_DATA_DIR "/lxpanel/images/clock.png");
+		}
         gtk_widget_show(dc->clock_icon);
         gtk_widget_hide(dc->clock_label);
     }
@@ -393,6 +398,15 @@ static void dclock_apply_configuration(Plugin * p)
         gtk_widget_show(dc->clock_label);
         gtk_widget_hide(dc->clock_icon);
     }
+    
+    if (dc->center_text)
+    {
+	    gtk_label_set_justify(GTK_LABEL(dc->clock_label), GTK_JUSTIFY_CENTER);
+    }
+    else
+    {
+	    gtk_label_set_justify(GTK_LABEL(dc->clock_label), GTK_JUSTIFY_LEFT);
+    }
 
     /* Rerun the experiment to determine update interval and update the display. */
     g_free(dc->prev_clock_value);
@@ -425,6 +439,7 @@ static void dclock_configure(Plugin * p, GtkWindow * parent)
         _("Action when clicked (default: display calendar)"), &dc->action, CONF_TYPE_STR,
         _("Bold font"), &dc->bold, CONF_TYPE_BOOL,
         _("Tooltip only"), &dc->icon_only, CONF_TYPE_BOOL,
+        _("Center text"), &dc->center_text, CONF_TYPE_BOOL,
         NULL);
     gtk_window_present(GTK_WINDOW(dlg));
 }
@@ -438,6 +453,7 @@ static void dclock_save_configuration(Plugin * p, FILE * fp)
     lxpanel_put_str(fp, "Action", dc->action);
     lxpanel_put_int(fp, "BoldFont", dc->bold);
     lxpanel_put_int(fp, "IconOnly", dc->icon_only);
+    lxpanel_put_int(fp, "CenterText", dc->center_text);
 }
 
 /* Callback when panel configuration changes. */
