@@ -131,8 +131,8 @@ menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
     int ox, oy, w, h;
     Plugin *p;
 #if GTK_CHECK_VERSION(2,18,0)
-    GtkAllocation allocation;
-    gtk_widget_set_allocation(widget, &allocation);
+    GtkAllocation *allocation = g_new0 (GtkAllocation, 1);
+    gtk_widget_get_allocation(GTK_WIDGET(widget), allocation);
 #endif
     ENTER;
     p = g_object_get_data(G_OBJECT(widget), "plugin");
@@ -155,20 +155,20 @@ menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
         *x = ox;
         if (*x + w > gdk_screen_width())
 #if GTK_CHECK_VERSION(2,18,0)
-            *x = ox + allocation.width - w;
+            *x = ox + allocation->width - w;
 #else
             *x = ox + widget->allocation.width - w;
 #endif
         *y = oy - h;
         if (*y < 0)
 #if GTK_CHECK_VERSION(2,18,0)
-            *y = oy + allocation.height;
+            *y = oy + allocation->height;
 #else
             *y = oy + widget->allocation.height;
 #endif
     } else {
 #if GTK_CHECK_VERSION(2,18,0)
-        *x = ox + allocation.width;
+        *x = ox + allocation->width;
 #else
         *x = ox + widget->allocation.width;
 #endif
@@ -177,19 +177,22 @@ menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
         *y = oy;
         if (*y + h >  gdk_screen_height())
 #if GTK_CHECK_VERSION(2,18,0)
-            *y = oy + allocation.height - h;
+            *y = oy + allocation->height - h;
 #else
             *y = oy + widget->allocation.height - h;
 #endif
     }
     DBG("widget: x,y=%d,%d  w,h=%d,%d\n", ox, oy,
 #if GTK_CHECK_VERSION(2,18,0)
-          allocation.width, allocation.height );
+          allocation->width, allocation->height );
 #else
           widget->allocation.width, widget->allocation.height );
 #endif
     DBG("w-h %d %d\n", w, h);
     *push_in = TRUE;
+#if GTK_CHECK_VERSION(2,18,0)
+    g_free (allocation);
+#endif
     RET();
 }
 
@@ -578,7 +581,7 @@ static void sys_menu_insert_items( menup* m, GtkMenu* menu, int position )
         /* add a place holder */
         GtkWidget* mi = gtk_menu_item_new();
         g_object_set_qdata( G_OBJECT(mi), SYS_MENU_ITEM_ID, GINT_TO_POINTER(1) );
-        gtk_menu_shell_insert(menu, mi, position);
+        gtk_menu_shell_insert(GTK_MENU_SHELL(menu), mi, position);
     }
 
     change_handler = g_signal_connect_swapped( gtk_icon_theme_get_default(), "changed", G_CALLBACK(unload_old_icons), menu );
@@ -593,7 +596,6 @@ reload_system_menu( menup* m, GtkMenu* menu )
     GtkMenuItem* item;
     GtkWidget* sub_menu;
     gint idx;
-    gboolean found = FALSE;
 
     children = gtk_container_get_children( GTK_CONTAINER(menu) );
     for( child = children, idx = 0; child; child = child->next, ++idx )
@@ -610,7 +612,6 @@ reload_system_menu( menup* m, GtkMenu* menu )
             sys_menu_insert_items( m, menu, idx );
             if( ! child )
                 break;
-            found = TRUE;
         }
         else if( ( sub_menu = gtk_menu_item_get_submenu( item ) ) )
         {
@@ -634,8 +635,8 @@ my_button_pressed(GtkWidget *widget, GdkEventButton *event, Plugin* plugin)
 {
     ENTER;
 #if GTK_CHECK_VERSION(2,18,0)
-    GtkAllocation allocation;
-    gtk_widget_get_allocation(widget, &allocation);
+    GtkAllocation *allocation = g_new0 (GtkAllocation, 1);
+    gtk_widget_get_allocation(GTK_WIDGET(widget), allocation);
 #endif
 
     /* Standard right-click handling. */
@@ -644,14 +645,17 @@ my_button_pressed(GtkWidget *widget, GdkEventButton *event, Plugin* plugin)
 
     if ((event->type == GDK_BUTTON_PRESS)
 #if GTK_CHECK_VERSION(2,18,0)
-          && (event->x >=0 && event->x < allocation.width)
-          && (event->y >=0 && event->y < allocation.height)) {
+          && (event->x >=0 && event->x < allocation->width)
+          && (event->y >=0 && event->y < allocation->height)) {
 #else
           && (event->x >=0 && event->x < widget->allocation.width)
           && (event->y >=0 && event->y < widget->allocation.height)) {
 #endif
         show_menu( widget, plugin, event->button, event->time );
     }
+#if GTK_CHECK_VERSION(2,18,0)
+    g_free (allocation);
+#endif
     RET(TRUE);
 }
 
@@ -1065,8 +1069,9 @@ static void save_config( Plugin* p, FILE* fp )
 static void apply_config(Plugin* p)
 {
     menup* m = (menup*)p->priv;
-    if( m->fname )
-        fb_button_set_from_file( m->img, m->fname, -1, p->panel->icon_size, TRUE );
+	if( m->fname ) {
+		fb_button_set_from_file( m->img, m->fname, -1, p->panel->icon_size, TRUE );
+	}
 }
 
 static void menu_config( Plugin *p, GtkWindow* parent )
