@@ -126,10 +126,9 @@ static gchar *taskbar_rc = "style 'taskbar-style'\n"
 
 #define DRAG_ACTIVE_DELAY    1000
 #define TASK_WIDTH_MAX       200
-#define TASK_PADDING         4
 #define ALL_WORKSPACES       0xFFFFFFFF		/* 64-bit clean */
 #define ICON_ONLY_EXTRA      6		/* Amount needed to have button lay out symmetrically */
-#define BUTTON_HEIGHT_EXTRA  4          /* Amount needed to have button not clip icon */
+#define ICON_BUTTON_TRIM 4		/* Amount needed to have button remain on panel */
 
 static void set_timer_on_task(Task * tk);
 static gboolean task_is_visible_on_current_desktop(TaskbarPlugin * tb, Task * tk);
@@ -438,6 +437,7 @@ static void task_unlink_class(Task * tk)
             if (tk_cursor == tk)
                 tk_pred->res_class_flink = tk->res_class_flink;
         }
+        tk->res_class_flink = NULL;
 
         /* Recompute group visibility. */
         recompute_group_visibility_for_class(tk->tb, tc);
@@ -553,8 +553,10 @@ static void task_delete(TaskbarPlugin * tb, Task * tk, gboolean unlink)
         tb->focused = NULL;
 
     /* If there is an urgency timeout, remove it. */
-    if (tk->flash_timeout != 0)
+    if (tk->flash_timeout != 0) {
         g_source_remove(tk->flash_timeout);
+        tk->flash_timeout = 0;
+    }
 
     /* Deallocate structures. */
     icon_grid_remove(tb->icon_grid, tk->button);
@@ -919,7 +921,7 @@ static GdkPixbuf * get_wm_icon(Window task_win, int required_width, int required
 static GdkPixbuf * task_update_icon(TaskbarPlugin * tb, Task * tk, Atom source)
 {
     /* Get the icon from the window's hints. */
-    GdkPixbuf * pixbuf = get_wm_icon(tk->win, tb->icon_size, tb->icon_size, source, &tk->image_source);
+    GdkPixbuf * pixbuf = get_wm_icon(tk->win, tb->icon_size - ICON_BUTTON_TRIM, tb->icon_size - ICON_BUTTON_TRIM, source, &tk->image_source);
 
     /* If that fails, and we have no other icon yet, return the fallback icon. */
     if ((pixbuf == NULL)
@@ -1241,7 +1243,7 @@ static void taskbar_update_style(TaskbarPlugin * tb)
 {
     GtkOrientation bo = (tb->plug->panel->orientation == ORIENT_HORIZ) ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
     icon_grid_set_geometry(tb->icon_grid, bo,
-        ((tb->icons_only) ? tb->icon_size + ICON_ONLY_EXTRA : tb->task_width_max), tb->icon_size + BUTTON_HEIGHT_EXTRA,
+        ((tb->icons_only) ? tb->icon_size + ICON_ONLY_EXTRA : tb->task_width_max), tb->icon_size,
         tb->spacing, 0, tb->plug->panel->height);
 }
 
@@ -2017,10 +2019,6 @@ static void taskbar_panel_configuration_changed(Plugin * p)
     TaskbarPlugin * tb = (TaskbarPlugin *) p->priv;
     taskbar_update_style(tb);
     taskbar_make_menu(tb);
-    GtkOrientation bo = (tb->plug->panel->orientation == ORIENT_HORIZ) ? GTK_ORIENTATION_HORIZONTAL : GTK_ORIENTATION_VERTICAL;
-    icon_grid_set_geometry(tb->icon_grid, bo,
-        ((tb->icons_only) ? tb->plug->panel->icon_size + ICON_ONLY_EXTRA : tb->task_width_max), tb->plug->panel->icon_size + BUTTON_HEIGHT_EXTRA,
-        tb->spacing, 0, tb->plug->panel->height);
 
     /* If the icon size changed, refetch all the icons. */
     if (tb->plug->panel->icon_size != tb->icon_size)
